# Робототехнической комплекс для гравировки-фрезеровки

Робототехнической комплекс (РТК) выполняет автоматизированную гравировку-фрезеровку шильдиков размером 40x60x2 мм. и весом 2-10 гр., но может быть настроен на работу с заготовками большего размера и веса (до 100 гр.).

## Концепция использования

Оператор комплекса (специалист ОТК или главный конструктор) выполняет отправку заданий на гравировку, вводя в веб-интерфейс информацию о содержании полей данных, наносимых на шильдик. После ввода информации создаётся спецификация задачи и отправляется в очередь задач управляющей программы РТК и осуществляется гравировка (подробный алгоритм которой изложен в разделе "Алгоритм работы").

## Типы шильдиков и наборы полей данных

| Тип устройства | Перечень параметров |
| - | - |
| Насос общепромышленный | Наименование, Q - подача, n - частота оборотов, H - напор, m - масса, P2 - мощность, T - диапазон температур, Дата производства, Заводской номер |
| Насос взрывозащищённый (Ex) | Наименование, Q - подача, n - частота оборотов, H - напор, m - масса, P2 - мощность, T - диапазон температур, Дата производства, Заводской номер, Диапазон ta в градусах цельсия |
| Насосная установка| Наименование, Q - подача, H - напор, m - масса, P - мощность, Кол-во насосов, Дата производства, Заводской номер |
| Шкаф управления   | Наименование, Iоб, Iсум, IP, Кол-во электродвигателей, Дата производства, Заводской номер |

## Состав системы

1. Робот-манипулятор Robossembler Arm 6-DoF с [вакуумным захватным устройством](https://gitlab.com/robossembler/arm-tools/vacuum-gripper) и камерой на запястье
2. Компьютер x64-совместимый с графическим ускорителем
3. Станок фрезеровки или лазерной гравировки
4. Кондуктор для размещения необработанных заготовок в станке
5. Приспособление для хранения заготовок шильдиков
6. Набор соединительных кабелей для CAN-шины и источника питания
7. Источник питания 24В, 15А
8. CAN-USB приёмопередатчик
9. Место укладки обработанных шильдиков

## Программное обеспечение

1. [Robossembler Framework](https://gitlab.com/robossembler/robossembler-ros2)
2. Robossembler Arm: ROS 2 driver & [Robot description](https://github.com/solid-sinusoid/rbs-arm)
3. Генератор векторных изображений чертежей шильдиков из текстовых описаний
4. Преобразователь векторных изображений в GCode или иной формат, который поддерживает станок
5. Модуль интеграции ROS 2 и ПО CNC/Graver
6. Веб-интерфейс для отправки заданий на гравировку

## Алгоритм работы

Управляющая программа построена на базе ROS 2 Jazzy и представляет собой дерево поведения из следующих основных компонентов:

### Узел обработки заданий на гравировку

- принимает от пользователя через пользовательский интерфейс данные об изделии, которые требуется наносить на шильдик
- формирует очередь задач
- для каждой из задач запускает навык гравировки
- отправляет в пользовательский вывод информацию о статусе завершения работы
- возвращается к очереди

### Поддерево исполнения навыка гравировки

- обнаружить требуемый шильдик
- добраться до шильдика и захватить
- установить шильдик в кондуктор в станке
- вернуться в предопределённую безопасную позицию
- отправить команду на исполнение станком гравировки
- дождаться сигнала от станка о выполнении гравировки
- добраться до шильдика в станке и захватить
- переместить в зону с готовыми шильдиками
- вернуться в начало

### Набор условий безопасности, позволяющий к экстренном режиме перейти в состояние аварийного останова

- аварийный рубильник
- предельные таймауты для каждой задачи (если задача не решена за N секунд, то вызывать оператора)
- машинное зрение для контроля выполнения задачи (например, контроль захвата шильдика, отрыв заготовки в процессе перемещения)
- перечень действий оператора для устранения неполадок

## Документация

1. Инструкция оператора РТК
2. Элекрическая схема установки
3. Руководство по настройке и эксплуатации РТК
4. Конструкторская документация на аппаратный комплекс
5. Исходные коды программ