#include "behaviortree_ros2/bt_service_node.hpp"
#include "behaviortree_ros2/plugins.hpp"
#include "rbs_utils_interfaces/srv/task_from_queue.hpp"

using TaskFromQueue = rbs_utils_interfaces::srv::TaskFromQueue;
using namespace BT;

class TaskCompleted : public RosServiceNode<TaskFromQueue> {
public:
TaskCompleted(const std::string &name, const NodeConfig &conf, const RosNodeParams &params)
      : RosServiceNode<TaskFromQueue>(name, conf, params) {

    // RCLCPP_INFO(this->logger(), "Starting TakeonTask");
  }

  static PortsList providedPorts() {
    return providedBasicPorts({});
  }

  bool setRequest(Request::SharedPtr &request) override {
    RCLCPP_INFO(this->logger(), "[%s] Starting send request for: [%s]",
                name().c_str(), this->service_name_.c_str());
    request->mode = "completed";
    return true;
  }

  NodeStatus onResponseReceived(const Response::SharedPtr &response) override {
    if (!response->ok) {
      RCLCPP_WARN(this->node_.lock()->get_logger(),
                   "[%s] Not completed", name().c_str());
      return NodeStatus::FAILURE;
    }

    RCLCPP_INFO(this->node_.lock()->get_logger(),
                "[%s] Task completed", name().c_str());
    return NodeStatus::SUCCESS;
  }
};

CreateRosNodePlugin(TaskCompleted, "TaskCompleted");
