#include "behaviortree_ros2/bt_action_node.hpp"
#include "geometry_msgs/msg/twist.hpp"
#include "rbs_skill_interfaces/action/twist_control_with_condition.hpp"
#include <behaviortree_cpp/tree_node.h>
#include <behaviortree_ros2/plugins.hpp>
#include <behaviortree_ros2/ros_node_params.hpp>
#include <rclcpp/logging.hpp>

using namespace BT;
using TwistCmdWithCondAction =
    rbs_skill_interfaces::action::TwistControlWithCondition;

class TwistCmdWithCondClient : public RosActionNode<TwistCmdWithCondAction> {
public:
  TwistCmdWithCondClient(const std::string &name, const NodeConfig &conf,
                         const RosNodeParams &params)
      : RosActionNode<TwistCmdWithCondAction>(name, conf, params) {

    RCLCPP_INFO(this->logger(), "Starting MoveToPose");
  }

  static BT::PortsList providedPorts() {
    return providedBasicPorts({BT::InputPort<std::string>("robot_name"),
                               BT::InputPort<std::vector<double>>("twist"),
                               // BT::InputPort<std::string>("condition_topic")
    });
  }

  bool setGoal(RosActionNode::Goal &goal) override {
    RCLCPP_INFO(this->logger(), "[%s] Starting send goal [%s]", name().c_str(),
                this->action_name_.c_str());

    if (!getInput("robot_name", goal.robot_name)) {
      RCLCPP_FATAL(this->logger(), "[%s] Could not get robot name",
                   name().c_str());
      return false;
    }

    if (!getInput("twist", twist_cmd_vec)) {
      RCLCPP_FATAL(this->logger(), "[%s] Could not get twist command",
                   name().c_str());
      return false;
    }
    if (!twist_cmd_vec.empty() && twist_cmd_vec.size() == 6) {
      goal.twist_cmd.linear.x = twist_cmd_vec[0];
      goal.twist_cmd.linear.y = twist_cmd_vec[1];
      goal.twist_cmd.linear.z = twist_cmd_vec[2];
      goal.twist_cmd.angular.x = twist_cmd_vec[3];
      goal.twist_cmd.angular.y = twist_cmd_vec[4];
      goal.twist_cmd.angular.z = twist_cmd_vec[5];
    } else {
      RCLCPP_FATAL(this->logger(),
                   "[%s] Twist command is empty or should be 1x6 [x;y;z;r;p;y]",
                   name().c_str());
      return false;
    }

    // if (!getInput("condition_topic", goal.condition_topic)) {
    //   RCLCPP_FATAL(this->logger(), "[%s] Could not get condition_topic",
    //                name().c_str());
    //   return false;
    // } else {
    //   RCLCPP_INFO(this->logger(), "[%s] Using condition_topic [%s]",
    //               name().c_str(), goal.condition_topic.c_str());
    // }

    return true;
  }

  NodeStatus onResultReceived(const WrappedResult &wr) override {

    RCLCPP_INFO(this->logger(), "[%s] Starting get response %s with status %b",
                name().c_str(), this->action_name_.c_str(), wr.result->success);
    if (!wr.result->success) {
      return NodeStatus::FAILURE;
    }
    return NodeStatus::SUCCESS;
  }

private:
  // std::shared_ptr<geometry_msgs::msg::Pose> m_target_pose;
  std::vector<double> twist_cmd_vec;
};

CreateRosNodePlugin(TwistCmdWithCondClient, "TwistCmdWithCond");
