#ifndef RBS_MILL_ASSIST__FULL_SYSTEM_HH_
#define RBS_MILL_ASSIST__FULL_SYSTEM_HH_

// The only required include in the header is this one.
// All others will depend on what your plugin does.
#include "gz/sim/components/Factory.hh"
#include <gz/sim/EventManager.hh>
#include <gz/sim/System.hh>

#include "std_srvs/srv/set_bool.hpp"
#include "std_msgs/msg/bool.hpp"
#include <memory>
#include <rclcpp/publisher.hpp>
#include <rclcpp/service.hpp>

namespace rbs_mill_assist {

class VacuumGripperPrivate;

class VacuumGripper : public gz::sim::System,
                      public gz::sim::ISystemConfigure,
                      public gz::sim::ISystemPreUpdate,
                      public gz::sim::ISystemPostUpdate {
public:
  VacuumGripper();
  ~VacuumGripper() override;
  void Configure(const gz::sim::Entity &_entity,
                 const std::shared_ptr<const sdf::Element> &_element,
                 gz::sim::EntityComponentManager &_ecm,
                 gz::sim::EventManager &_eventManager) override;

  void PreUpdate(const gz::sim::UpdateInfo &_info,
                 gz::sim::EntityComponentManager &_ecm) override;

  void PostUpdate(const gz::sim::UpdateInfo &_info,
                  const gz::sim::EntityComponentManager &_ecm) override;

  bool AttachLink(gz::sim::EntityComponentManager &_ecm,
                  const gz::sim::Entity &gripperEntity,
                  const gz::sim::Entity &objectEntity);

  bool DetachLink(gz::sim::EntityComponentManager &_ecm);

  void Toggle(const std::shared_ptr<std_srvs::srv::SetBool::Request> request,
              std::shared_ptr<std_srvs::srv::SetBool::Response> response);

private:
  std::unique_ptr<VacuumGripperPrivate> dataPtr;
  rclcpp::Service<std_srvs::srv::SetBool>::SharedPtr toggle_service_;
  rclcpp::Publisher<std_msgs::msg::Bool>::SharedPtr contact_publisher_;
};
} // namespace rbs_mill_assist
#endif
