#include "geometry_msgs/msg/quaternion.hpp"
#include "rbs_mill_interfaces/action/add_collision_object.hpp"
#include "rbs_mill_interfaces/action/check_object_exists.hpp"
#include "rbs_mill_interfaces/action/remove_collision_object.hpp"
#include "rbs_mill_assist/places_config.hpp"
#include <geometry_msgs/msg/pose.hpp>
#include <memory>
#include <rclcpp/rclcpp.hpp>
#include <rclcpp_action/client.hpp>
#include <tuple>
#include <yaml-cpp/node/node.h>

class CollisionSpawner : public rclcpp::Node {

public:
  struct ObjectSpec {
    std::string name;
    std::string type;
    std::string frame_id = "world";
    bool has_dimensions;
    std::vector<double> dimensions;
    geometry_msgs::msg::Pose pose;
    std::optional<std::string> mesh_filepath;
    std::tuple<double, double, double> scale_xyz = {1.0, 1.0, 1.0};
  };

  struct Objects {
    std::optional<std::string> model_dir;
    std::vector<ObjectSpec> models;
  };

  using AddCollisionObject = rbs_mill_interfaces::action::AddCollisionObject;
  using RemoveCollisionObject =
      rbs_mill_interfaces::action::RemoveCollisionObject;
  using CheckObjectExists = rbs_mill_interfaces::action::CheckObjectExists;

  CollisionSpawner();

private:
  void removeExistingObjects();

  void sendRemoveGoal(const std::string &object_id);
  void spawnObjects();

  void sendAddGoal(const PlacesConfig::PlaceSpec &obj_spec);

  bool isObjectExist(const std::string &object_id);

  std::shared_ptr<PlacesConfig> _places_config;

  rclcpp_action::Client<AddCollisionObject>::SharedPtr
      add_object_action_client_;
  rclcpp_action::Client<RemoveCollisionObject>::SharedPtr
      remove_object_action_client_;
  rclcpp_action::Client<CheckObjectExists>::SharedPtr
      check_object_exists_action_client_;
  std::string config_file_;
};
